@@ -17,9 +17,9 @@ from simple_parsing.utils import (Dataclass, SimpleValueType,
                                   get_type_arguments, is_optional, is_tuple,
                                   is_union, str2bool)
 
-from ..logging_utils import get_logger
+from logging import getLogger
 
-logger = get_logger(__file__)
+logger = getLogger(__name__)
 
 E = TypeVar("E", bound=Enum)
 K = TypeVar("K")
@@ -155,10 +155,6 @@ def field(default: Union[T, _MISSING_TYPE] = MISSING,
         )
 
 
-@overload
-def choice(*choices: T, default: T=None, **kwargs) -> T:
-    pass
-
 @overload
 def choice(choices: Type[E], default: E, **kwargs) -> E:
     pass
@@ -167,7 +163,7 @@ def choice(choices: Type[E], default: E, **kwargs) -> E:
 def choice(choices: Dict[K, V], default: K, **kwargs) -> V:
     pass
 
-def choice(*choices: T, default: T = None, **kwargs: Any) -> T:
+def choice(*choices: T, default: Union[T, _MISSING_TYPE] = MISSING, **kwargs: Any) -> T:
     """ Makes a field which can be chosen from the set of choices from the
     command-line.
 
@@ -179,7 +175,7 @@ def choice(*choices: T, default: T = None, **kwargs: Any) -> T:
     Similarly for Enum types, passing a type of enum will  
 
     Args:
-        default (T, optional): The default value of the field. Defaults to None,
+        default (T, optional): The default value of the field. Defaults to dataclasses.MISSING,
         in which case the command-line argument is required.
 
     Raises:
@@ -198,7 +194,7 @@ def choice(*choices: T, default: T = None, **kwargs: Any) -> T:
             choices = OrderedDict(
                 (e.name, e) for e in choice_enum
             )
-            if default is not None and not isinstance(default, choice_enum):
+            if default is not MISSING and not isinstance(default, choice_enum):
                 if default in choices:
                     warnings.warn(UserWarning(
                         f"Setting default={default} could perhaps be ambiguous "
@@ -213,10 +209,33 @@ def choice(*choices: T, default: T = None, **kwargs: Any) -> T:
             # if the choices is a dict, the options are the keys
             # save the info about the choice_dict in the field metadata.
             metadata = kwargs.setdefault("metadata", {})
+            choice_dict = choices
             # save the choice_dict in metadata so that we can recover the values in postprocessing.
-            metadata["choice_dict"] = choices
-            choices = list(choices.keys())
+            metadata["choice_dict"] = choice_dict
+            choices = list(choice_dict.keys())
+        
+            # TODO: If the choice dict is given, then add encoding/decoding functions that just
+            # get/set the right key.
+            def _encoding_fn(value: Any) -> str:
+                """ Custom encoding function that will simply represent the value as the
+                the key in the dict rather than the value itself.
+                """
+                if value in choice_dict.keys():
+                    return value
+                elif value in choice_dict.values():
+                    return [k for k, v in choice_dict.items() if v == value][0]
+                return value        
+            kwargs.setdefault("encoding_fn", _encoding_fn)
+            
+            def _decoding_fn(value: Any) -> str:
+                """ Custom decoding function that will retrieve the value from the
+                stored key in the dictionary.
+                """
+                return choice_dict.get(value, value)
+    
+            kwargs.setdefault("decoding_fn", _decoding_fn)
 
+            
     return field(
         default=default,
         choices=choices,
