@@ -6,14 +6,14 @@ import warnings
 from collections import OrderedDict
 from dataclasses import Field, fields, is_dataclass
 from functools import lru_cache, partial
+from logging import getLogger
 from typing import *
 
-from ...logging_utils import get_logger
 from ...utils import (get_item_type, get_type_arguments, is_dict, is_list,
                       is_set, is_tuple, is_union)
 
-logger = get_logger(__file__)
-# logger.setLevel(logging.DEBUG)
+logger = getLogger(__name__)
+
 T = TypeVar("T")
 K = TypeVar("K")
 V = TypeVar("V")
@@ -115,6 +115,10 @@ def get_decoding_fn(t: Type[T]) -> Callable[[Any], T]:
     elif is_list(t):
         logger.debug(f"Decoding a List field: {t}")
         args = get_type_arguments(t)
+        if not args:
+            # Using a `List` or `list` annotation, so we don't know what do decode the
+            # items into!
+            args = (Any,)
         assert len(args) == 1
         return decode_list(args[0])
 
@@ -181,7 +185,7 @@ def try_functions(*funcs: Callable[[Any], T]) -> Callable[[Any], Union[T, Any]]:
             except Exception as ex:
                 e = ex
         else:
-            logger.error(f"Couldn't parse value {val}, returning it as-is. (exception: {e})")
+            logger.debug(f"Couldn't parse value {val}, returning it as-is. (exception: {e})")
         return val
     return _try_functions
 
@@ -218,18 +222,29 @@ def decode_tuple(*tuple_item_types: Type[T]) -> Callable[[List[T]], Tuple[T, ...
         Callable[[List[T]], Tuple[T, ...]]: A parsing function for creating tuples.
     """
     # Get the decoding function for each item type
-    decoding_fns = [
-        get_decoding_fn(t) for t in tuple_item_types
-    ]
+    has_ellipsis = False
+    if Ellipsis in tuple_item_types:
+        # TODO: This isn't necessary, the ellipsis will always be at index 1.
+        ellipsis_index = tuple_item_types.index(Ellipsis)
+        decoding_fn_index = ellipsis_index - 1
+        decoding_fn = get_decoding_fn(tuple_item_types[decoding_fn_index])
+        has_ellipsis = True
+    else:
+        decoding_fns = [
+            get_decoding_fn(t) for t in tuple_item_types
+        ]
     # Note, if there are more values than types in the tuple type, then the
     # last type is used.
-    # TODO: support the Ellipsis?
-
     def _decode_tuple(val: Tuple[Any, ...]) -> Tuple[T, ...]:
         result: List[T] = []
-        return tuple(
-            decoding_fns[i](v) for i, v in enumerate(val)
-        )
+        if has_ellipsis:
+            return tuple(
+                decoding_fn(v) for v in val
+            )
+        else:
+            return tuple(
+                decoding_fns[i](v) for i, v in enumerate(val)
+            )
     return _decode_tuple
 
 
@@ -269,6 +284,10 @@ def decode_dict(K_: Type[K], V_: Type[V]) -> Callable[[List[Tuple[Any, Any]]], D
         if isinstance(val, list):
             result = OrderedDict()
             items = val
+        elif isinstance(val, OrderedDict):
+            # NOTE(ycho): Needed to propagate `OrderedDict` type
+            result = OrderedDict()
+            items = val.items()
         else:
             items = val.items()
         for k, v in items:
