@@ -5,19 +5,19 @@ import warnings
 from collections import OrderedDict
 from dataclasses import MISSING, Field, asdict, dataclass, fields, is_dataclass
 from functools import singledispatch
+from logging import getLogger
 from pathlib import Path
 from typing import *
 from typing import IO, TypeVar
 
 import typing_inspect as tpi
 
-from ...logging_utils import get_logger
 from ...utils import get_type_arguments, is_dict, is_list, is_union
 from .decoding import (_decoding_fns, decode_field, get_decoding_fn,
                        register_decoding_fn)
 from .encoding import SimpleJsonEncoder, encode
 
-logger = get_logger(__file__)
+logger = getLogger(__name__)
 
 Dataclass = TypeVar("Dataclass")
 D = TypeVar("D", bound="Serializable")
@@ -25,11 +25,13 @@ D = TypeVar("D", bound="Serializable")
 try:
     import yaml
     def ordered_dict_constructor(loader: yaml.Loader, node: yaml.Node):
-        value = loader.construct_sequence(node)
+        # NOTE(ycho): `deep` has to be true for `construct_yaml_seq`.
+        value = loader.construct_sequence(node, deep = True)
         return OrderedDict(*value)
 
     def ordered_dict_representer(dumper: yaml.Dumper, instance: OrderedDict) -> yaml.Node:
-        node = dumper.represent_sequence("OrderedDict", instance.items())
+        # NOTE(ycho): nested list for compatibility with PyYAML's representer
+        node = dumper.represent_sequence("OrderedDict", [list(instance.items())])
         return node
 
     yaml.add_representer(OrderedDict, ordered_dict_representer)
@@ -57,8 +59,9 @@ class Serializable:
     >>> config
     Config(a=123, b='456')
     >>> config.to_dict()
-    {"a": 123, "b": 456}
+    {'a': 123, 'b': '456'}
     >>> config_ = Config.from_dict({"a": 123, "b": 456})
+    >>> config_
     Config(a=123, b='456')
     >>> assert config == config_
     """
@@ -115,7 +118,10 @@ class Serializable:
 
             encoding_fn = encode
             if isinstance(value, Serializable) and recurse:
-                encoded = value.to_dict(dict_factory=dict_factory, recurse=recurse)
+                try:
+                    encoded = value.to_dict(dict_factory=dict_factory, recurse=recurse)
+                except TypeError as e:
+                    encoded = value.to_dict()
                 logger.debug(f"Encoded Serializable field {name}: {encoded}")
             else:
                 try:
@@ -512,6 +518,7 @@ def from_dict(cls: Type[Dataclass], d: Dict[str, Any], drop_extra_fields: bool=N
     return instance
 
 
+# TODO: Remove, unused.
 def get_key_and_value_types(dict_type: Type[Dict], Serializable=Serializable) -> Tuple[Optional[Type], Optional[Type]]:
     args = get_type_arguments(dict_type)
 
