@@ -1,29 +1,69 @@
 """Utility functions used in various parts of the simple_parsing package."""
 import argparse
 import builtins
+import inspect
+import enum
 import dataclasses
 import functools
 import json
 import re
 import warnings
+import itertools
+import hashlib
 from abc import ABC
+from collections import OrderedDict
 from collections import abc as c_abc
 from collections import defaultdict
 from dataclasses import _MISSING_TYPE, MISSING, Field, dataclass
 from enum import Enum
 from functools import partial
 from inspect import isclass
-from typing import (Any, Callable, Container, Dict, Iterable, List, Mapping,
-                    MutableMapping, Optional, Set, Tuple, Type, TypeVar, Union)
+from logging import getLogger
+from typing import (
+    Any,
+    Callable,
+    Container,
+    Dict,
+    Iterable,
+    List,
+    Mapping,
+    MutableMapping,
+    Optional,
+    Set,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+)
 
 import typing_inspect as tpi
 
-from simple_parsing.logging_utils import get_logger
 
-logger = get_logger(__file__)
+try:
+    from typing import get_args
+except ImportError:
+    # try:
+    #     # TODO: Not sure we should depend on typing_inspect, results appear to vary
+    #     # greatly
+    #     # between python versions.
+    #     from typing_inspect import get_args
+    # except ImportError:
+    def get_args(some_type: Type) -> Tuple[Type, ...]:
+        return getattr(some_type, "__args__", ())
 
-builtin_types = [getattr(builtins, d) for d in dir(
-    builtins) if isinstance(getattr(builtins, d), type)]
+try:
+    from typing import get_origin
+except ImportError:
+    from typing_inspect import get_origin
+
+
+logger = getLogger(__name__)
+
+builtin_types = [
+    getattr(builtins, d)
+    for d in dir(builtins)
+    if isinstance(getattr(builtins, d), type)
+]
 
 K = TypeVar("K")
 T = TypeVar("T")
@@ -35,9 +75,9 @@ Dataclass = TypeVar("Dataclass")
 DataclassType = Type[Dataclass]
 
 SimpleValueType = Union[bool, int, float, str]
-SimpleIterable = Union[List[SimpleValueType],
-                       Dict[Any, SimpleValueType], Set[SimpleValueType]]
-
+SimpleIterable = Union[
+    List[SimpleValueType], Dict[Any, SimpleValueType], Set[SimpleValueType]
+]
 
 
 def is_subparser_field(field: Field) -> bool:
@@ -57,12 +97,12 @@ class InconsistentArgumentError(RuntimeError):
 
 
 def camel_case(name):
-    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
-    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()
+    s1 = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", name)
+    return re.sub("([a-z0-9])([A-Z])", r"\1_\2", s1).lower()
 
 
-TRUE_STRINGS: List[str] = ['yes', 'true', 't', 'y', '1']
-FALSE_STRINGS: List[str] = ['no', 'false', 'f', 'n', '0']
+TRUE_STRINGS: List[str] = ["yes", "true", "t", "y", "1"]
+FALSE_STRINGS: List[str] = ["no", "false", "f", "n", "0"]
 
 
 def str2bool(raw_value: Union[str, bool]) -> bool:
@@ -78,17 +118,18 @@ def str2bool(raw_value: Union[str, bool]) -> bool:
         return False
     else:
         raise argparse.ArgumentTypeError(
-            f"Boolean value expected for argument, received '{raw_value}'")
+            f"Boolean value expected for argument, received '{raw_value}'"
+        )
 
 
 def get_item_type(container_type: Type[Container[T]]) -> T:
     """Returns the `type` of the items in the provided container `type`.
-    
+
     When no type annotation is found, or no item type is found, returns
     `typing.Any`.
     NOTE: If a type with multiple arguments is passed, only the first type
     argument is returned.
-    
+
     >>> import typing
     >>> from typing import List, Tuple
     >>> get_item_type(list)
@@ -124,7 +165,17 @@ def get_item_type(container_type: Type[Container[T]]) -> T:
     Returns:
         Type -- the type of the container's items, if found, else Any.
     """
-    if container_type in {list, set, tuple, List, Set, Tuple, Dict, Mapping, MutableMapping}:
+    if container_type in {
+        list,
+        set,
+        tuple,
+        List,
+        Set,
+        Tuple,
+        Dict,
+        Mapping,
+        MutableMapping,
+    }:
         # the built-in `list` and `tuple` types don't have annotations for their item types.
         return Any
     type_arguments = getattr(container_type, "__args__", None)
@@ -134,7 +185,9 @@ def get_item_type(container_type: Type[Container[T]]) -> T:
         return Any
 
 
-def get_argparse_type_for_container(container_type: Type) -> Union[Type, Callable[[str], bool]]:
+def get_argparse_type_for_container(
+    container_type: Type,
+) -> Union[Type, Callable[[str], bool]]:
     """Gets the argparse 'type' option to be used for a given container type.
     When an annotation is present, the 'type' option of argparse is set to that type.
     if not, then the default value of 'str' is returned.
@@ -144,6 +197,10 @@ def get_argparse_type_for_container(container_type: Type) -> Union[Type, Callabl
 
     Returns:
         typing.Type -- the type that should be used in argparse 'type' argument option.
+    
+    TODO: This overlaps in a weird way with `get_parsing_fn`, which returns the 'type'
+    to use for a given annotation! This function however doesn't deal with 'weird' item
+    types, it just returns the first annotation.
     """
     T = get_item_type(container_type)
     if T is bool:
@@ -154,6 +211,10 @@ def get_argparse_type_for_container(container_type: Type) -> Union[Type, Callabl
 
 
 def _mro(t: Type) -> List[Type]:
+    # TODO: This is mostly used in 'is_tuple' and such, and should be replaced with
+    # either the build-in 'get_origin' from typing, or from typing-inspect.
+    if t is None:
+        return []
     if hasattr(t, "__mro__"):
         return t.__mro__
     elif tpi.get_origin(t) is type:
@@ -171,7 +232,7 @@ def is_list(t: Type) -> bool:
 
     Returns:
         bool: True if `t` is list or a subclass of list.
-    
+
     >>> from typing import *
     >>> is_list(list)
     True
@@ -202,7 +263,7 @@ def is_tuple(t: Type) -> bool:
 
     Returns:
         bool: True if `t` is tuple or a subclass of tuple.
-    
+
     >>> from typing import *
     >>> is_tuple(list)
     False
@@ -219,6 +280,8 @@ def is_tuple(t: Type) -> bool:
     ...
     >>> is_tuple(foo)
     True
+    >>> is_tuple(List[int])
+    False
     """
     return tuple in _mro(t)
 
@@ -231,7 +294,7 @@ def is_dict(t: Type) -> bool:
 
     Returns:
         bool: True if `t` is dict or a subclass of dict.
-    
+
     >>> from typing import *
     >>> from collections import OrderedDict
     >>> is_dict(dict)
@@ -268,7 +331,7 @@ def is_set(t: Type) -> bool:
 
     Returns:
         bool: True if `t` is set or a subclass of set.
-    
+
     >>> from typing import *
     >>> is_set(set)
     True
@@ -294,7 +357,6 @@ def is_set(t: Type) -> bool:
     return set in _mro(t)
 
 
-
 def is_dataclass_type(t: Type) -> bool:
     """Returns wether t is a dataclass type or a TypeVar of a dataclass type.
 
@@ -310,6 +372,8 @@ def is_dataclass_type(t: Type) -> bool:
 
 
 def is_enum(t: Type) -> bool:
+    if inspect.isclass(t):
+        return issubclass(t, enum.Enum)
     return Enum in _mro(t)
 
 
@@ -341,19 +405,59 @@ def is_union(t: Type) -> bool:
     return getattr(t, "__origin__", "") == Union
 
 
+def is_homogeneous_tuple_type(t: Type[Tuple]) -> bool:
+    """Returns wether the given Tuple type is homogeneous: if all items types are the
+    same.
+
+    This also includes Tuple[<some_type>, ...]
+
+    Returns
+    -------
+    bool
+
+    >>> from typing import *
+    >>> is_homogeneous_tuple_type(Tuple)
+    True
+    >>> is_homogeneous_tuple_type(Tuple[int, int])
+    True
+    >>> is_homogeneous_tuple_type(Tuple[int, str])
+    False
+    >>> is_homogeneous_tuple_type(Tuple[int, str, float])
+    False
+    >>> is_homogeneous_tuple_type(Tuple[int, ...])
+    True
+    >>> is_homogeneous_tuple_type(Tuple[Tuple[int, str], ...])
+    True
+    >>> is_homogeneous_tuple_type(Tuple[List[int], List[str]])
+    False
+    """
+    if not is_tuple(t):
+        return False
+    type_arguments = get_type_arguments(t)
+    if not type_arguments:
+        return True
+    assert isinstance(type_arguments, tuple), type_arguments
+    if len(type_arguments) == 2 and type_arguments[1] is Ellipsis:
+        return True
+    # Tuple[str, str, str] -> True
+    # Tuple[str, str, float] -> False
+    # TODO: Not sure if this will work with more complex item times (like nested tuples)
+    return len(set(type_arguments)) == 1
+
+
 def is_choice(field: Field) -> bool:
     return bool(field.metadata.get("custom_args", {}).get("choices", {}))
 
 
 def is_optional(t: Type) -> bool:
     """Returns True if the given Type is a variant of the Optional type.
-    
+
     Parameters
     ----------
     - t : Type
-    
+
         a Type annotation (or "live" type)
-    
+
     Returns
     -------
     bool
@@ -384,10 +488,7 @@ def contains_dataclass_type_arg(t: Type) -> bool:
     elif is_tuple_or_list_of_dataclasses(t):
         return True
     elif is_union(t):
-        return any(
-            contains_dataclass_type_arg(arg)
-            for arg in get_type_arguments(t)
-        )
+        return any(contains_dataclass_type_arg(arg) for arg in get_type_arguments(t))
     return False
 
 
@@ -397,15 +498,18 @@ def get_dataclass_type_arg(t: Type) -> Optional[Type]:
     if is_dataclass_type(t):
         return t
     elif is_tuple_or_list(t) or is_union(t):
-        return next(filter(None, (
-            get_dataclass_type_arg(arg)
-            for arg in get_type_arguments(t)
-        )), None)
+        return next(
+            filter(
+                None, (get_dataclass_type_arg(arg) for arg in get_type_arguments(t))
+            ),
+            None,
+        )
     return None
 
 
-def get_type_arguments(container_type: Type) -> List[Type]:
-    return getattr(container_type, "__args__", [])
+def get_type_arguments(container_type: Type) -> Tuple[Type, ...]:
+    # return getattr(container_type, "__args__", ())
+    return get_args(container_type)
 
 
 def get_type_name(some_type: Type):
@@ -417,27 +521,66 @@ def get_type_name(some_type: Type):
 
 
 def get_container_nargs(container_type: Type) -> Union[int, str]:
+    """Gets the value of 'nargs' appropriate for the given container type.
+    
+    Parameters
+    ----------
+    container_type : Type
+        Some container type.
+
+    Returns
+    -------
+    Union[int, str]
+        [description]
+    """
     if is_tuple(container_type):
-        type_arguments = getattr(container_type, "__args__", [])
-        if type_arguments and Ellipsis not in type_arguments:
-            nargs = len(type_arguments)
-            # if nargs == 1:
-            #     # a `Tuple[int]` annotation can be interpreted as "a tuple of an unknown number of ints"?.
-            #     return "*"
-            return nargs
+        # TODO: Should a `Tuple[int]` annotation be interpreted as "a tuple of an
+        # unknown number of ints"?.
+        type_arguments: Tuple[Type, ...] = get_type_arguments(container_type)
+        if not type_arguments:
+            return "*"
+        if len(type_arguments) == 2 and type_arguments[1] is Ellipsis:
+            return "*"
 
-    return "*"
+        total_nargs: int = 0
+        for item_type in type_arguments:
+            # TODO: Handle the 'nargs' for nested container types!
+            if is_list(item_type) or is_tuple(item_type):
+                # BUG: If it's a container like Tuple[Tuple[int, str], Tuple[int, str]]
+                # we could do one of two things:
+                # 
+                # - Option 1: Use nargs=4 and re-organize/split values in
+                #   post-processing.
+                # item_nargs: Union[int, str] = get_container_nargs(item_type)
+                # if isinstance(item_nargs, int):
+                #     total_nargs += item_nargs
+                # else:
+                #     return "*"
+                # 
+                # This is a bit confusing, and IMO it might be best to just do
+                # - Option 2: Use `nargs='*'` and use a custom parsing function that
+                #   will convert entries appropriately..
+                return "*"
+            total_nargs += 1
+        return total_nargs
+
+    if is_list(container_type):
+        return "*"
+    raise NotImplementedError(
+        f"Not sure what 'nargs' should be for type {container_type}"
+    )
 
 
-def _parse_multiple_containers(container_type: type, append_action: bool = False) -> Callable[[str], List[Any]]:
+def _parse_multiple_containers(
+    container_type: type, append_action: bool = False
+) -> Callable[[str], List[Any]]:
     T = get_argparse_type_for_container(container_type)
     factory = tuple if is_tuple(container_type) else list
 
     result = factory()
 
     def parse_fn(value: str):
-        logger.debug(
-            f"parsing multiple {container_type} of {T}s, value is: '{value}'")
+        logger.debug(f"parsing multiple {container_type} of {T}s, value is: '{value}'")
         values = _parse_container(container_type)(value)
         logger.debug(f"parsing result is '{values}'")
 
@@ -446,6 +589,7 @@ def _parse_multiple_containers(container_type: type, append_action: bool = False
             return result
         else:
             return values
+
     return parse_fn
 
 
@@ -462,7 +606,8 @@ def _parse_container(container_type: Type[Container]) -> Callable[[str], List[An
             values = _parse_literal(value)
         except Exception as e:
             logger.debug(
-                f"Exception while trying to parse '{value}' as a literal: {type(e)}: {e}")
+                f"Exception while trying to parse '{value}' as a literal: {type(e)}: {e}"
+            )
             # if it doesnt work, fall back to the parse_fn.
             values = _fallback_parse(value)
 
@@ -472,7 +617,7 @@ def _parse_container(container_type: Type[Container]) -> Callable[[str], List[An
         return values
 
     def _parse_literal(value: str) -> Union[List[Any], Any]:
-        """ try to parse the string to a python expression directly.
+        """try to parse the string to a python expression directly.
         (useful for nested lists or tuples.)
         """
         literal = ast.literal_eval(value)
@@ -487,7 +632,7 @@ def _parse_container(container_type: Type[Container]) -> Callable[[str], List[An
             return values
 
     def _fallback_parse(v: str) -> List[Any]:
-        v = ' '.join(v.split())
+        v = " ".join(v.split())
         if v.startswith("[") and v.endswith("]"):
             v = v[1:-1]
 
@@ -527,9 +672,7 @@ def get_nesting_level(possibly_nested_list):
     elif len(possibly_nested_list) == 0:
         return 1
     else:
-        return 1 + max(
-            get_nesting_level(item) for item in possibly_nested_list
-        )
+        return 1 + max(get_nesting_level(item) for item in possibly_nested_list)
 
 
 def default_value(field: dataclasses.Field) -> Union[T, _MISSING_TYPE]:
@@ -545,7 +688,7 @@ def default_value(field: dataclasses.Field) -> Union[T, _MISSING_TYPE]:
     if field.default is not dataclasses.MISSING:
         return field.default
     elif field.default_factory is not dataclasses.MISSING:  # type: ignore
-        constructor = field.default_factory # type: ignore
+        constructor = field.default_factory  # type: ignore
         return constructor()
     else:
         return dataclasses.MISSING
@@ -559,7 +702,7 @@ def trie(sentences: List[List[str]]) -> Dict[str, Union[str, Dict]]:
 
     Returns:
         Dict[str, Union[str, Dict[str, ...]]]: A tree where each node is a word in a sentence.
-        Sentences which begin with the same words share the first nodes, etc. 
+        Sentences which begin with the same words share the first nodes, etc.
     """
     first_word_to_sentences: Dict[str, List[List[str]]] = defaultdict(list)
     for sentence in sentences:
@@ -571,8 +714,7 @@ def trie(sentences: List[List[str]]) -> Dict[str, Union[str, Dict]]:
         if len(sentences) == 1:
             return_dict[first_word] = ".".join(sentences[0])
         else:
-            sentences_without_first_word = [
-                sentence[1:] for sentence in sentences]
+            sentences_without_first_word = [sentence[1:] for sentence in sentences]
             return_dict[first_word] = trie(sentences_without_first_word)
     return return_dict
 
@@ -602,6 +744,105 @@ def keep_keys(d: Dict, keys_to_keep: Iterable[str]) -> Tuple[Dict, Dict]:
     return d, removed
 
 
+def compute_identity(size: int = 16, **sample) -> str:
+    """Compute a unique hash out of a dictionary
+
+    Parameters
+    ----------
+    size: int
+        size of the unique hash
+
+    **sample:
+        Dictionary to compute the hash from
+
+    """
+    sample_hash = hashlib.sha256()
+
+    for k, v in sorted(sample.items()):
+        sample_hash.update(k.encode("utf8"))
+
+        if isinstance(v, dict):
+            sample_hash.update(compute_identity(size, **v).encode("utf8"))
+        else:
+            sample_hash.update(str(v).encode("utf8"))
+
+    return sample_hash.hexdigest()[:size]
+
+
+def dict_intersection(*dicts: Dict[K, V]) -> Iterable[Tuple[K, Tuple[V, ...]]]:
+    common_keys = set(dicts[0])
+    for d in dicts:
+        common_keys.intersection_update(d)
+    for key in common_keys:
+        yield (key, tuple(d[key] for d in dicts))
+
+
+def field_dict(dataclass: Dataclass) -> Dict[str, Field]:
+    result: Dict[str, Field] = OrderedDict()
+    for field in dataclasses.fields(dataclass):
+        result[field.name] = field
+    return result
+
+
+def zip_dicts(*dicts: Dict[K, V]) -> Iterable[Tuple[K, Tuple[Optional[V], ...]]]:
+    # If any attributes are common to both the Experiment and the State,
+    # copy them over to the Experiment.
+    keys = set(itertools.chain(*dicts))
+    for key in keys:
+        yield (key, tuple(d.get(key) for d in dicts))
+
+
+def dict_union(
+    *dicts: Dict[K, V], recurse: bool = True, dict_factory=dict
+) -> Dict[K, V]:
+    """Simple dict union until we use python 3.9
+
+    If `recurse` is True, also does the union of nested dictionaries.
+    NOTE: The returned dictionary has keys sorted alphabetically.
+    >>> from collections import OrderedDict
+    >>> a = OrderedDict(a=1, b=2, c=3)
+    >>> b = OrderedDict(c=5, d=6, e=7)
+    >>> dict_union(a, b, dict_factory=OrderedDict)
+    OrderedDict([('a', 1), ('b', 2), ('c', 5), ('d', 6), ('e', 7)])
+    >>> a = OrderedDict(a=1, b=OrderedDict(c=2, d=3))
+    >>> b = OrderedDict(a=2, b=OrderedDict(c=3, e=6))
+    >>> dict_union(a, b, dict_factory=OrderedDict)
+    OrderedDict([('a', 2), ('b', OrderedDict([('c', 3), ('d', 3), ('e', 6)]))])
+    """
+    result: Dict = dict_factory()
+    if not dicts:
+        return result
+    assert len(dicts) >= 1
+    all_keys: Set[str] = set()
+    all_keys.update(*dicts)
+    all_keys = sorted(all_keys)
+
+    # Create a neat generator of generators, to save some memory.
+    all_values: Iterable[Tuple[V, Iterable[K]]] = (
+        (k, (d[k] for d in dicts if k in d)) for k in all_keys
+    )
+    for k, values in all_values:
+        sub_dicts: List[Dict] = []
+        new_value: V = None
+        n_values = 0
+        for v in values:
+            if isinstance(v, dict) and recurse:
+                sub_dicts.append(v)
+            else:
+                # Overwrite the new value for that key.
+                new_value = v
+            n_values += 1
+
+        if len(sub_dicts) == n_values and recurse:
+            # We only get here if all values for key `k` were dictionaries,
+            # and if recurse was True.
+            new_value = dict_union(*sub_dicts, recurse=True, dict_factory=dict_factory)
+
+        result[k] = new_value
+    return result
+
+
 if __name__ == "__main__":
     import doctest
+
     doctest.testmod()
