@@ -1,6 +1,7 @@
 import argparse
 import dataclasses
 import enum
+from logging import getLogger
 from typing import *
 from typing import cast
 from dataclasses import _MISSING_TYPE, MISSING
@@ -8,9 +9,8 @@ from .. import docstring, utils
 from ..utils import Dataclass, DataclassType
 from .wrapper import Wrapper
 from .field_wrapper import FieldWrapper
-from ..logging_utils import get_logger
 
-logger = get_logger(__file__)
+logger = getLogger(__name__)
 
 class DataclassWrapper(Wrapper[Dataclass]):
     def __init__(self,
@@ -24,7 +24,8 @@ class DataclassWrapper(Wrapper[Dataclass]):
         # super().__init__(dataclass, name)
         self.dataclass = dataclass
         self._name = name
-        self.default = default
+        self.default = default 
+        self.prefix = prefix
 
         self.fields: List[FieldWrapper] = []
         self._destinations: List[str] = []
@@ -46,7 +47,7 @@ class DataclassWrapper(Wrapper[Dataclass]):
 
 
         for field in dataclasses.fields(self.dataclass):
-            if not field.init:
+            if not field.init or field.metadata.get("cmd", True) == False:
                 continue
 
             if utils.is_subparser_field(field) or utils.is_choice(field):
@@ -75,7 +76,7 @@ class DataclassWrapper(Wrapper[Dataclass]):
 
             else:
                 # a normal attribute
-                field_wrapper = FieldWrapper(field, parent=self)
+                field_wrapper = FieldWrapper(field, parent=self, prefix=self.prefix)
                 logger.debug(f"wrapped field at {field_wrapper.dest} has a default value of {field_wrapper.default}")
                 self.fields.append(field_wrapper)
         
